@@ -58,7 +58,7 @@ pmt <- function(key, ...) {
 #' 
 #' @return a data frame containing keys and corresponding tests implemented in this package.
 #' 
-#' @examples pmts()
+#' @examples pmts("ksample")
 #' 
 #' @export
 
@@ -95,20 +95,26 @@ pmts <- function(
 
 #' @rdname pmt
 #' 
+#' @param inherit a character string specifying the type of permutation test.
 #' @param statistic definition of the test statistic. See Details.
-#' @param inherit a character string specifying the desired permutation test.
 #' @param rejection a character string specifying where the rejection region is.
-#' @param scoring,n_permu passed to the constructor.
+#' @param scoring one of:
+#'      - a character string in `c("none", "rank", "vw", "expon")` specifying the scoring system
+#'      - a function that takes a numeric vector and returns an equal-length score vector
+#' @param n_permu an integer indicating number of permutations for the permutation distribution. If set to `0`, all permutations will be used.
 #' @param name a character string specifying the name of the test.
-#' @param alternative a character string specifying the alternative of the test.
+#' @param alternative a character string describing the alternative hypothesis.
 #' @param depends,plugins,includes passed to [Rcpp::cppFunction()].
 #' 
-#' @details The test statistic in `define_pmt` can be defined using either `R` or `Rcpp`, with the `statistic` parameter specified as:
+#' @return a test object based on the specified statistic.
+#' 
+#' @details
+#' The test statistic can be defined using either `R` or `Rcpp`, with the `statistic` parameter specified as:
 #' 
 #' - `R`: a function returning a closure that returns a double.
-#' - `Rcpp`: a character string defining a captureless lambda (introduced in C++11) returning another lambda that may capture by value, accepts const arguments of the same type, and returns a double.
+#' - `Rcpp`: a character string defining a captureless lambda (since C++11) returning another lambda that captures by value, accepts parameters of the same type as const references, and returns a double.
 #' 
-#' When using `Rcpp`, the parameters for different `inherit` are listed as follows. Note that the parameter names are illustrative and may be modified.
+#' When using `Rcpp`, the parameters for different `inherit` are listed as follows. Note that the parameter names are for illustration only.
 #' 
 #' - `"twosample"`: `(Rcpp::NumericVector sample_1, Rcpp::NumericVector sample_2)`
 #' - `"ksample"`: `(Rcpp::NumericVector combined_sample, Rcpp::IntegerVector one_based_group_index)`
@@ -120,9 +126,19 @@ pmts <- function(
 #' Defining the test statistic using `R` follows a similar approach. The purpose of this design is to pre-calculate certain constants that remain invariant during permutation.
 #' 
 #' @examples
+#' x <- rnorm(100)
+#' y <- rnorm(100, 1)
+#' 
+#' t <- define_pmt(
+#'     inherit = "twosample",
+#'     scoring = base::rank, # equivalent to "rank"
+#'     statistic = function(...) function(x, y) sum(x)
+#' )$test(x, y)$print()
+#' 
 #' \donttest{
 #' r <- define_pmt(
-#'     inherit = "twosample", rejection = "lr", n_permu = 1e5,
+#'     inherit = "twosample",
+#'     n_permu = 1e5,
 #'     statistic = function(x, y) {
 #'         m <- length(x)
 #'         n <- length(y)
@@ -130,43 +146,36 @@ pmts <- function(
 #'     }
 #' )
 #' 
-#' cpp <- define_pmt(
-#'     inherit = "twosample", rejection = "lr", n_permu = 1e5,
+#' rcpp <- define_pmt(
+#'     inherit = "twosample",
+#'     n_permu = 1e5,
 #'     statistic = "[](NumericVector x, NumericVector y) {
-#'         R_len_t n_x = x.size();
-#'         R_len_t n_y = y.size();
-#'         return [n_x, n_y](const NumericVector x, const NumericVector y) {
-#'             double sum_x = 0;
-#'             double sum_y = 0;
-#'             for(auto x_i : x) sum_x += x_i;
-#'             for(auto y_i : y) sum_y += y_i;
-#'             return sum_x / n_x - sum_y / n_y;
+#'         R_len_t m = x.size();
+#'         R_len_t n = y.size();
+#'         return [=](const NumericVector& x, const NumericVector& y) -> double {
+#'             return sum(x) / m - sum(y) / n;
 #'         };
 #'     }"
 #' )
 #' 
-#' x <- rnorm(100)
-#' y <- rnorm(100, 1)
 #' options(LearnNonparam.pmt_progress = FALSE)
 #' system.time(r$test(x, y))
-#' system.time(cpp$test(x, y))
+#' system.time(rcpp$test(x, y))
 #' }
 #' 
 #' @export
 #' 
 #' @importFrom R6 R6Class
-#' @importFrom Rcpp cppFunction
+#' @importFrom Rcpp cppFunction evalCpp
 #' @importFrom compiler cmpfun
 
 define_pmt <- function(
-    statistic,
     inherit = c(
-        "twosample", "ksample",
-        "paired", "rcbd",
-        "association", "table"
+        "twosample", "ksample", "paired", "rcbd", "association", "table"
     ),
+    statistic,
     rejection = c("lr", "l", "r"),
-    scoring = c("none", "rank", "vw", "expon"), n_permu = 1e4,
+    scoring = "none", n_permu = 1e4,
     name = "User-Defined Permutation Test", alternative = NULL,
     depends = character(), plugins = character(), includes = character()
 ) {
@@ -190,7 +199,8 @@ define_pmt <- function(
             table = ContingencyTableTest
         ),
         public = list(
-            initialize = function(n_permu) {
+            initialize = function(scoring, n_permu) {
+                self$scoring <- scoring
                 self$n_permu <- n_permu
 
                 if (typeof(statistic) == "closure") {
@@ -202,25 +212,28 @@ define_pmt <- function(
                     cppFunction(
                         env = environment(super$.calculate_statistic),
                         depends = c(depends, "LearnNonparam"),
-                        plugins = unique(c(plugins, "cpp14")),
+                        plugins = {
+                            cpp_standard_ver <- evalCpp("__cplusplus")
+                            c(plugins, if (cpp_standard_ver < 201402L) "cpp14")
+                        },
                         includes = {
                             hpps <- c("progress", "reorder", impl)
-                            c(includes, paste0("#include <pmt/", hpps, ".hpp>"))
+                            c(includes, paste0("#include<pmt/", hpps, ".hpp>"))
                         },
                         code = {
-                            n <- if (inherit == "rcbd") 2 else 3
-                            args <- paste0("arg_", 1:n)
+                            args <- paste0(
+                                "arg", 1:(n <- if (inherit == "rcbd") 2 else 3)
+                            )
                             paste0(
                                 "SEXP ", inherit, "_pmt(",
                                 paste("SEXP", args, collapse = ","),
                                 ", double n_permu, bool progress){",
                                 "auto statistic = ", statistic, ";",
                                 "return progress ?", paste0(
-                                    impl, "<PermuBar",
-                                    c("Show", "Hide"), ">(",
+                                    impl, "<PermuBar", c("Show", "Hide"), ">(",
                                     paste0(
-                                        "clone(", args[-n], ")", collapse = ", "
-                                    ), ", statistic, n_permu)", collapse = " : "
+                                        "clone(", args[-n], ")", collapse = ","
+                                    ), ", statistic, n_permu )", collapse = ":"
                                 ), ";}"
                             )
                         }
@@ -231,7 +244,7 @@ define_pmt <- function(
         private = list(
             .name = if (!missing(name)) as.character(name) else name,
             .alternative = if (!missing(alternative)) as.character(alternative),
-            .scoring = match.arg(scoring),
+
             .side = match.arg(rejection),
 
             .calculate = function() {
@@ -245,6 +258,34 @@ define_pmt <- function(
                 private$.calculate_n_permu()
                 private$.calculate_p_permu()
             }
+        ),
+        active = list(
+            scoring = function(value) {
+                if (missing(value)) {
+                    return(private$.scoring)
+                } else if (is.character(value)) {
+                    private$.scoring <- match.arg(
+                        value, c("none", "rank", "vw", "expon")
+                    )
+                } else if (is.function(value)) {
+                    private$.scoring <- "custom"
+                    assign(
+                        envir = environment(super$.calculate_score),
+                        "get_score", function(x, ...) {
+                            score <- value(x)
+                            if (
+                                is.numeric(score) && length(score) == length(x)
+                            ) score else stop("Invalid scoring system")
+                        }
+                    )
+                } else {
+                    stop("'scoring' must be a character string or a function")
+                }
+
+                if (!is.null(private$.raw_data)) {
+                    private$.on_scoring_change()
+                }
+            }
         )
-    )$new(n_permu)
+    )$new(scoring, n_permu)
 }
\ No newline at end of file
